package vn.hus.nlp.fsm;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public abstract class FSM
{
  protected Map<Integer, State> states = new HashMap();
  protected Map<Integer, List<Transition>> intransitionMap;
  private int nTransitions;

  public FSM()
  {
    createInitialState();
    this.nTransitions = 0;
    this.intransitionMap = new HashMap();
  }

  private void createInitialState()
  {
    State localState = new State(0);
    localState.setType((byte)0);
    this.states.put(Integer.valueOf(0), localState);
  }

  public void addState(State paramState)
  {
    this.states.put(Integer.valueOf(paramState.getId()), paramState);
  }

  public void removeState(State paramState)
  {
    if (paramState.getId() <= 0)
      return;
    List localList = paramState.getOutTransitions();
    this.nTransitions -= localList.size();
    localList.clear();
    this.states.remove(Integer.valueOf(paramState.getId()));
  }

  private void addIntransitionMap(Transition paramTransition)
  {
    List<Transition> localObject = (List<Transition>)this.intransitionMap.get(Integer.valueOf(paramTransition.getTarget()));
    if (localObject == null)
    {
      localObject = new ArrayList();
      this.intransitionMap.put(Integer.valueOf(paramTransition.getTarget()), localObject);
    }
    ((List)localObject).add(paramTransition);
  }

  private void removeIntransitionMap(Transition paramTransition)
  {
    List localList = (List)this.intransitionMap.get(Integer.valueOf(paramTransition.getTarget()));
    localList.remove(paramTransition);
  }

  public void addTransition(Transition paramTransition)
  {
    if (paramTransition != null)
    {
      State localState = (State)this.states.get(Integer.valueOf(paramTransition.getSource()));
      localState.getOutTransitions().add(paramTransition);
      this.nTransitions += 1;
      addIntransitionMap(paramTransition);
    }
  }

  public void removeTransition(Transition paramTransition)
  {
    if (paramTransition != null)
    {
      State localState = (State)this.states.get(Integer.valueOf(paramTransition.getSource()));
      localState.getOutTransitions().remove(paramTransition);
      this.nTransitions -= 1;
      removeIntransitionMap(paramTransition);
    }
  }

  public Map<Integer, State> getStates()
  {
    return this.states;
  }

  public State getState(int paramInt)
  {
    if ((paramInt < 0) || (paramInt > this.states.size()))
      return null;
    return (State)this.states.get(new Integer(paramInt));
  }

  public Transition getTransition(int paramInt1, int paramInt2)
  {
    State localState = getState(paramInt1);
    List localList = localState.getOutTransitions();
    Iterator localIterator = localList.iterator();
    while (localIterator.hasNext())
    {
      Transition localTransition = (Transition)localIterator.next();
      if (localTransition.getTarget() == paramInt2)
        return localTransition;
    }
    return null;
  }

  public State getInitialState()
  {
    return (State)this.states.get(new Integer(0));
  }

  public int getNTransitions()
  {
    return this.nTransitions;
  }

  public Map<Integer, List<Transition>> getIntransitionMap()
  {
    return this.intransitionMap;
  }

  public State getNextState(State paramState, char paramChar)
  {
    Iterator localIterator = paramState.getOutTransitions().iterator();
    while (localIterator.hasNext())
    {
      Transition localTransition = (Transition)localIterator.next();
      if (localTransition.getInput() == paramChar)
        return getState(localTransition.getTarget());
    }
    return null;
  }

  public String getNextOutput(State paramState, char paramChar)
  {
    Iterator localIterator = paramState.getOutTransitions().iterator();
    while (localIterator.hasNext())
    {
      Transition localTransition = (Transition)localIterator.next();
      if (localTransition.getInput() == paramChar)
        return localTransition.getOutput();
    }
    return "";
  }

  public abstract ISimulator getSimulator();

  public void dispose()
  {
    this.states.clear();
    this.states = null;
    this.intransitionMap.clear();
    this.intransitionMap = null;
  }
}

/* Location:           C:\Downloads\vn.hus.nlp.tokenizer-4.1.1-bin.tar\lib\vn.hus.nlp.fsm-1.0.0.jar
 * Qualified Name:     vn.hus.nlp.fsm.FSM
 * JD-Core Version:    0.6.0
 */